#!/usr/bin/python
# -*- coding: utf-8 -*-

#from __future__ import unicode_literals
# Import used modules:
import glob
#import os
import re
from xml.dom import minidom
import sys
#debugging:
import helper_functions as h
#import make_options as o

# TODO: there are still empty lines in the whole_corpus ...

def _get_text(nodelist):
    ''' list -> string
    extract text from nodelist elements and returns it
    '''
    text = ""
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            text += node.data
        else:
            node_type = h.return_nodetype(node)
            h.print_error("Can't get data from %s, type %s code %d"
                    % (node, node_type, node.nodeType))
    return text

def _get_tag_value(node):
    ''' node -> string
    get a node and flatten it, so inline tags are removed
    process it, so various parts are deleted
    and return the cleaned string
    '''
    # flattening the element to string
    xml_str = node.toxml('utf-8').decode('utf-8')
    # removing inline tags
    xml_str = re.sub(ur'<.*?>', u'', xml_str)
    # removing leftover braces 
    xml_str = re.sub(ur'[<>]', u'', xml_str)
    # removing multiple whitespaces
    xml_str = re.sub(r'[ ]+', u' ', xml_str)
    # removing empty listpoints
    # TODO: why is the list empty?
    #xml_str = re.sub(r'[-]+', u'', xml_str)
    # removing "(+++ stuff +++)"
    xml_str = re.sub(r'\(\+{3}.*?\+{3}\)', u'', xml_str)
    # removing surrounding space
    return xml_str.strip()

def _get_content(dom):
    ''' dom-object -> list
    read xml content from a dom element, based on the name of the TagName
    (Content in this case). Append it to a list and return it.
    '''

    # find content
    content_nodes = dom.getElementsByTagName('Content')
    content_list = []
    for elem in content_nodes:
        text = _get_tag_value(elem)
        #text_list = re.split(' ', text)
        if text != "":# and isinstance(text_list, list) and len(text_list) > 4:
            content_list.append(text)

    # TODO: get metadata?
    # actually, no we don't want metadata
    #content_nodes = dom.getElementsByTagName('metadaten')
    #for elem in content_nodes:
    #    text = _get_tag_value(elem)
    #    if text != "":
    #        content_list.append(text)

    return content_list

def _get_name(dom):
    ''' dom-object -> string
    extract corpus name and abbreviation so it is possible to have some metadata
    '''
    # find name, abbreviation and content:
    name = dom.getElementsByTagName('langue')[0].childNodes
    name = _get_text(name)
    abbr = dom.getElementsByTagName('jurabk')[0].childNodes
    abbr = " (" + _get_text(abbr) + ")"
    return name + abbr

def _get_tokens(sents):
    ''' list -> list
    Tokenize Sentences
    '''
    tokens = []
    for line in sents:
        clean_token = [ re.sub('\w[0-9]{1,}\)', '', i) for i in line.split(' ') ]
        clean_token = [ i.strip(' -,.()[]{}_\'"') for i in clean_token ]
        clean_token = [ i for i in clean_token if not ( i == u'' or i == u'\n' or i == None) ]
        tokens.append(clean_token)
    return tokens

def _get_sents(content):
    ''' string -> list
    Sentence segementation
    '''
    # parse corpus in sentences and tokens:
    # return [ [first sentence with words as list elements], [second sentence ...] ]
    sents = []

    ### internal notice: think about the difference between re.split and str.splitlines
    ###content = re.split('\n+', content, re.U)
    content = content.splitlines()
    ###unsplittable = []
    ###for line in content:
    ###    if re.search('\n', line, re.U):
    ###        unsplittable.append(line)
    ###        subline = re.split('\n', line, re.U)
    ###        for line in subline:
    ###            if isinstance(line, list):
    ###                print line[0]
    ###                print subline.index(line)
    ###print len(unsplittable)
    ###h.write_file(unsplittable, 'unsplittable')
    for line in content:
        if line == u'':
            continue
        this_sentence = re.split(r'(?<!(\) [a-zA-Z0-9]|ABl|. S|hst|chn|Abs|Art|..[0-9]|. d|vgl|. v| Nr))[\.!?]', line)
        # documentation for the heart of sentence-splitting:
        #(?<!( the beginning of the negative look behind assertion

        # TODO: matches some abbreviation, this is questionable:
        # all questionable as well. they are overrepresentated in the shortest 'sentences':
        #..a ..n ..v ..z ..d

        #\) [a-zA-Z0-9]| matches sublist items like 1) a. or 1) 1.
        #ABl| ABl.L an abbreviation for some law
        #. S| the second part of ABl
        #hst| abbreviation for Buchstabe (Buchst.)
        #chn| is the abbreviation for Abschnitt (Abschn.)
        #Abs| matches abbreviation for Absatz ( Abs.)
        #Art| matches abbreviation for Artikel ( Art. )
        #..[0-9]| matches Dates or other numerals (08.05.1945 or 5.)
        #. d| matches abbreviation für der or den ( d.)
        #vgl| abbreviation for vergleiche (vgl.)
        #. v| abbreviation for von? (v.)
        #Nr)) abbreviation for Nummer (Nr.)
        #[\.!?]', line)

        # for basestring see:
        # http://docs.python.org/2/library/functions.html#basestring
        if isinstance(this_sentence, basestring) and this_sentence != u'':
            this_sentence = [this_sentence.strip()]
        for sent in this_sentence:
            # sent can be empty if there are multiple newlines
            if sent == u'' or sent == None:
                continue
            if re.match(r'[\s\W]*[0-9]*[\s\W]*$', sent):
                continue
            if isinstance(sent, basestring):
                sent = sent.strip()
                # there is most likely no meaningful sentence with less than 5 characters
                # (i tested this :))
                if len(sent) < 5:
                    # TODO: improve sentence segmentation:
                    #print_debug(sent)
                    continue
            sents.append(sent)
    return sents

def read_corpus():
    ''' () -> list
    Read the corpus from the specified location (h.corpus_dir in helper_functions.py)
    and process it with various other functions. extract xml-elements, extract text from
    xml elements. parse sentences. parse tokens. put everything in a nice datastructure
    return it
    '''
    h.print_verbose("Parsing corpora")
    corpus_files = glob.glob(h.corpus_dir + '*.xml')
    corpus_list = []
    start = 0
    for corpus_file in corpus_files:
        corpus_id = re.sub(r'^./corpus/', '', corpus_file)
        h.print_verbose("Working with %s" % corpus_id)

        h.print_verbose("Getting name and content", True)
        # read from the corpus, get name and content (list)
        dom = minidom.parse(corpus_file)
        name = _get_name(dom)
        content = _get_content(dom)

        # join the elements with newlines
        content = '\n'.join(content)
        # there are already newlines in the text so this joining will be split
        # up later again. This is much simpler than checking all the list
        # elements for newlines.

        h.print_verbose("Starting sentence segmentation", True)
        # sentence segmentation: return a list of sentences
        sents = _get_sents(content)

        #if o.options.debug:
        #    h.write_file(sents, h.temp_dir + 'testsentences')
        #    break

        h.print_verbose("Starting tokenization", True)
        # tokenziation: one list per sentence, elements are tokens
        tokens = _get_tokens(sents)

        # TODO: count tokens and add them to start
        # define end
        # corpus_token = sum([ len(sentence) for sentence in corpus_content ])
        length = sum([ len(token_list) for token_list in tokens ])
        end = start + length

        corpus_list.append({ 'id': corpus_id, 'name': name, 'start':start, 'end':end, 'tokenized_sents':tokens})
        start = end
    #h.exit_handler()
    return corpus_list

if __name__ == "__main__":
    h.print_error("Do not call this script directly")
    sys.exit(1)
